@@ -11,7 +11,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tkEntry.c,v 1.3.4.1 1999/04/24 00:10:49 stanton Exp $
+ * RCS: @(#) $Id: tkEntry.c,v 1.5.4.1 1999/11/19 23:35:44 hobbs Exp $
  */
 
 #include "tkInt.h"
@@ -1405,14 +1405,17 @@ DisplayEntry(clientData)
 		entryPtr->borderWidth, entryPtr->relief);
     }
     if (entryPtr->highlightWidth != 0) {
-	GC gc;
+	GC fgGC, bgGC;
 
+	bgGC = Tk_GCForColor(entryPtr->highlightBgColorPtr, pixmap);
 	if (entryPtr->flags & GOT_FOCUS) {
-	    gc = Tk_GCForColor(entryPtr->highlightColorPtr, pixmap);
+	    fgGC = Tk_GCForColor(entryPtr->highlightColorPtr, pixmap);
+	    TkpDrawHighlightBorder(tkwin, fgGC, bgGC, 
+	            entryPtr->highlightWidth, pixmap);
 	} else {
-	    gc = Tk_GCForColor(entryPtr->highlightBgColorPtr, pixmap);
+	    TkpDrawHighlightBorder(tkwin, bgGC, bgGC, 
+	            entryPtr->highlightWidth, pixmap);
 	}
-	Tk_DrawFocusHighlight(tkwin, gc, entryPtr->highlightWidth, pixmap);
     }
 
     /*
@@ -1943,7 +1946,7 @@ EntryCmdDeletedProc(clientData)
      * destroys the widget.
      */
 
-    if (! entryPtr->flags & ENTRY_DELETED) {
+    if (!(entryPtr->flags & ENTRY_DELETED)) {
         Tk_DestroyWindow(entryPtr->tkwin);
     }
 }
