@@ -7,12 +7,12 @@
  *	attributes.
  *
  * Copyright (c) 1990-1994 The Regents of the University of California.
- * Copyright (c) 1994-1995 Sun Microsystems, Inc.
+ * Copyright (c) 1994-1997 Sun Microsystems, Inc.
  *
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * SCCS: @(#) tkFrame.c 1.82 97/08/08 17:26:26
+ * RCS: @(#) $Id: tkFrame.c,v 1.4 1999/08/10 05:06:01 jingham Exp $
  */
 
 #include "default.h"
@@ -62,9 +62,6 @@ typedef struct {
     int highlightWidth;		/* Width in pixels of highlight to draw
 				 * around widget when it has the focus.
 				 * 0 means don't draw a highlight. */
-    Tk_Tile tile;		/* tiling */
-    Tk_TSOffset tsoffset;	/* offset for tiling */
-    GC tileGC;			/* GC for tiling */
     XColor *highlightBgColorPtr;
 				/* Color for drawing traversal highlight
 				 * area when highlight is off. */
@@ -100,22 +97,6 @@ typedef struct {
 #define REDRAW_PENDING		1
 #define GOT_FOCUS		4
 
-/*
- * Custom option for handling "-tile"
- */
-
-static Tk_CustomOption tileOption = {
-    (Tk_OptionParseProc *) Tk_TileParseProc,
-    Tk_TilePrintProc,
-    (ClientData) NULL
-};
-
-static Tk_CustomOption offsetOption = {
-    (Tk_OptionParseProc *) Tk_OffsetParseProc,
-    Tk_OffsetPrintProc,
-    (ClientData) NULL
-};
-
 /*
  * The following flag bits are used so that there can be separate
  * defaults for some configuration options for frames and toplevels.
@@ -162,9 +143,6 @@ static Tk_ConfigSpec configSpecs[] = {
     {TK_CONFIG_STRING, "-menu", "menu", "Menu",
 	DEF_TOPLEVEL_MENU, Tk_Offset(Frame, menuName),
 	TOPLEVEL|TK_CONFIG_NULL_OK},
-    {TK_CONFIG_CUSTOM, "-offset", "offset", "Offset", "0,0",
-	Tk_Offset(Frame, tsoffset),BOTH|TK_CONFIG_DONT_SET_DEFAULT,
-	&offsetOption},
     {TK_CONFIG_RELIEF, "-relief", "relief", "Relief",
 	DEF_FRAME_RELIEF, Tk_Offset(Frame, relief), BOTH},
     {TK_CONFIG_STRING, "-screen", "screen", "Screen",
@@ -173,8 +151,6 @@ static Tk_ConfigSpec configSpecs[] = {
     {TK_CONFIG_STRING, "-takefocus", "takeFocus", "TakeFocus",
 	DEF_FRAME_TAKE_FOCUS, Tk_Offset(Frame, takeFocus),
 	BOTH|TK_CONFIG_NULL_OK},
-    {TK_CONFIG_CUSTOM, "-tile", "tile", "Tile", (char *) NULL,
-	Tk_Offset(Frame, tile),BOTH|TK_CONFIG_DONT_SET_DEFAULT, &tileOption},
     {TK_CONFIG_STRING, "-use", "use", "Use",
 	DEF_FRAME_USE, Tk_Offset(Frame, useThis), TOPLEVEL|TK_CONFIG_NULL_OK},
     {TK_CONFIG_STRING, "-visual", "visual", "Visual",
@@ -202,9 +178,6 @@ static void		FrameEventProc _ANSI_ARGS_((ClientData clientData,
 static int		FrameWidgetCmd _ANSI_ARGS_((ClientData clientData,
 			    Tcl_Interp *interp, int argc, char **argv));
 static void		MapFrame _ANSI_ARGS_((ClientData clientData));
-static void		TileChangedProc _ANSI_ARGS_((ClientData clientData,
-			    Tk_Tile tile, Tk_Item *itemPtr));
-
 
 /*
  *--------------------------------------------------------------
@@ -432,11 +405,6 @@ TkCreateFrame(clientData, interp, argc, argv, toplevel, appName)
     framePtr->borderWidth = 0;
     framePtr->relief = TK_RELIEF_FLAT;
     framePtr->highlightWidth = 0;
-    framePtr->tile = NULL;
-    framePtr->tsoffset.flags = 0;
-    framePtr->tsoffset.xoffset = 0;
-    framePtr->tsoffset.yoffset = 0;
-    framePtr->tileGC = NULL;
     framePtr->highlightBgColorPtr = NULL;
     framePtr->highlightColorPtr = NULL;
     framePtr->width = 0;
@@ -473,7 +441,7 @@ TkCreateFrame(clientData, interp, argc, argv, toplevel, appName)
     if (toplevel) {
 	Tcl_DoWhenIdle(MapFrame, (ClientData) framePtr);
     }
-    interp->result = Tk_PathName(new);
+    Tcl_SetResult(interp, Tk_PathName(new), TCL_STATIC);
     return TCL_OK;
 
     error:
@@ -612,40 +580,12 @@ DestroyFrame(memPtr)
 
     Tk_FreeOptions(configSpecs, (char *) framePtr, framePtr->display,
 	    framePtr->mask);
-    if (framePtr->tile != NULL) {
-	Tk_FreeTile(framePtr->tile);
-    }
-    if (framePtr->tileGC != None) {
-	Tk_FreeGC(framePtr->display, framePtr->tileGC);
-    }
     if (framePtr->colormap != None) {
 	Tk_FreeColormap(framePtr->display, framePtr->colormap);
     }
     ckfree((char *) framePtr);
 }
 
-/*
- *----------------------------------------------------------------------
- *
- * TileChangedProc
- *
- * Results:
- *	None.
- *
- *----------------------------------------------------------------------
- */
-/* ARGSUSED */
-static void
-TileChangedProc(clientData, tile, itemPtr)
-    ClientData clientData;
-    Tk_Tile tile;
-    Tk_Item *itemPtr;			/* Not used */
-{
-    register Frame *framePtr = (Frame *) clientData;
-
-    ConfigureFrame(framePtr->interp, framePtr, 0, NULL, 0);
-}
-
 /*
  *----------------------------------------------------------------------
  *
@@ -657,7 +597,7 @@ TileChangedProc(clientData, tile, itemPtr)
  *
  * Results:
  *	The return value is a standard Tcl result.  If TCL_ERROR is
- *	returned, then interp->result contains an error message.
+ *	returned, then the interp's result contains an error message.
  *
  * Side effects:
  *	Configuration information, such as text string, colors, font,
@@ -677,9 +617,6 @@ ConfigureFrame(interp, framePtr, argc, argv, flags)
     int flags;			/* Flags to pass to Tk_ConfigureWidget. */
 {
     char *oldMenuName;
-    GC new = None;
-    XGCValues gcValues;
-    Pixmap pixmap;
     
     /*
      * Need the old menubar name for the menu code to delete it.
@@ -705,23 +642,12 @@ ConfigureFrame(interp, framePtr, argc, argv, flags)
 		framePtr->menuName);
     }
     
-    Tk_SetTileChangedProc(framePtr->tile, TileChangedProc,
-	(ClientData)framePtr, (Tk_Item *) NULL);
-    if ((pixmap = Tk_PixmapOfTile(framePtr->tile)) != None) {
-	gcValues.fill_style = FillTiled;
-	gcValues.tile = pixmap;
-	new = Tk_GetGC(framePtr->tkwin, GCTile|GCFillStyle, &gcValues);
-    } else if (framePtr->border != NULL) {
+    if (framePtr->border != NULL) {
 	Tk_SetBackgroundFromBorder(framePtr->tkwin, framePtr->border);
     } else {
 	Tk_SetWindowBackgroundPixmap(framePtr->tkwin, None);
     }
 
-    if (framePtr->tileGC != None) {
-	Tk_FreeGC(framePtr->display, framePtr->tileGC);
-    }
-    framePtr->tileGC = new;
-
     if (framePtr->highlightWidth < 0) {
 	framePtr->highlightWidth = 0;
     }
@@ -768,9 +694,6 @@ DisplayFrame(clientData)
 {
     register Frame *framePtr = (Frame *) clientData;
     register Tk_Window tkwin = framePtr->tkwin;
-    GC gc;
-    void (* drawFunction) _ANSI_ARGS_((Tk_Window, Drawable, Tk_3DBorder,
-	    int, int, int, int, int, int)) = Tk_Fill3DRectangle;
 
     framePtr->flags &= ~REDRAW_PENDING;
     if ((framePtr->tkwin == NULL) || !Tk_IsMapped(tkwin)
@@ -778,51 +701,8 @@ DisplayFrame(clientData)
 	return;
     }
 
-    if (framePtr->tileGC != NULL) {
-	int borderWidth = framePtr->highlightWidth;
-	if (framePtr->border != NULL) {
-	    borderWidth += framePtr->borderWidth;
-	}
-	if ((Tk_Width(tkwin) > (2 * borderWidth)) &&
-		(Tk_Height(tkwin) > (2 * borderWidth))) {
-	    Tk_SetTileOrigin(tkwin, framePtr->tileGC, 0, 0);
-	    if (framePtr->tsoffset.flags) {
-		int w=0; int h=0;
-		if (framePtr->tsoffset.flags &
-			(TK_OFFSET_CENTER|TK_OFFSET_MIDDLE)) {
-		    Tk_SizeOfTile(framePtr->tile, &w, &h);
-		}
-		if (framePtr->tsoffset.flags & TK_OFFSET_LEFT) {
-		    w = 0;
-		} else if (framePtr->tsoffset.flags & TK_OFFSET_RIGHT) {
-		    w = Tk_Width(tkwin);
-		} else {
-		    w = (Tk_Width(tkwin) - w) / 2;
-		}
-		if (framePtr->tsoffset.flags & TK_OFFSET_TOP) {
-		    h = 0;
-		} else if (framePtr->tsoffset.flags & TK_OFFSET_BOTTOM) {
-		    h = Tk_Height(tkwin);
-		} else {
-		    h = (Tk_Height(tkwin) - h) / 2;
-		}
-		XSetTSOrigin(framePtr->display, framePtr->tileGC, w , h);
-	    } else {
-	        Tk_SetTileOrigin(tkwin, framePtr->tileGC,
-			framePtr->tsoffset.xoffset,
-			framePtr->tsoffset.yoffset);
-	    }
-	    XFillRectangle(framePtr->display, Tk_WindowId(tkwin),
-		    framePtr->tileGC, borderWidth, borderWidth,
-		    Tk_Width(tkwin) - 2 * borderWidth,
-		    Tk_Height(tkwin) - 2 * borderWidth);
-	    XSetTSOrigin(framePtr->display, framePtr->tileGC, 0, 0);
-	    drawFunction = Tk_Draw3DRectangle;
-	}
-    }
-
     if (framePtr->border != NULL) {
-	drawFunction(tkwin, Tk_WindowId(tkwin),
+	Tk_Fill3DRectangle(tkwin, Tk_WindowId(tkwin),
 		framePtr->border, framePtr->highlightWidth,
 		framePtr->highlightWidth,
 		Tk_Width(tkwin) - 2*framePtr->highlightWidth,
@@ -830,15 +710,19 @@ DisplayFrame(clientData)
 		framePtr->borderWidth, framePtr->relief);
     }
     if (framePtr->highlightWidth != 0) {
+        GC fgGC, bgGC;
+        
+	bgGC = Tk_GCForColor(framePtr->highlightBgColorPtr,
+		Tk_WindowId(tkwin));
 	if (framePtr->flags & GOT_FOCUS) {
-	    gc = Tk_GCForColor(framePtr->highlightColorPtr,
+	    fgGC = Tk_GCForColor(framePtr->highlightColorPtr,
+		    Tk_WindowId(tkwin));
+	    TkpDrawHighlightBorder(tkwin, fgGC, bgGC, framePtr->highlightWidth,
 		    Tk_WindowId(tkwin));
 	} else {
-	    gc = Tk_GCForColor(framePtr->highlightBgColorPtr,
+	    TkpDrawHighlightBorder(tkwin, bgGC, bgGC, framePtr->highlightWidth,
 		    Tk_WindowId(tkwin));
 	}
-	Tk_DrawFocusHighlight(tkwin, gc, framePtr->highlightWidth,
-		Tk_WindowId(tkwin));
     }
 }
 
