@@ -11,7 +11,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * SCCS: @(#) tkListbox.c 1.120 97/10/29 13:06:59
+ * RCS: @(#) $Id: tkListbox.c,v 1.4 1999/08/10 05:06:47 jingham Exp $
  */
 
 #include "tkPort.h"
@@ -24,7 +24,7 @@
  */
 
 typedef struct Element {
-    int textLength;		/* # non-NULL characters in text. */
+    int textLength;		/* # non-NULL bytes in text string. */
     int lBearing;		/* Distance from first character's
 				 * origin to left edge of character. */
     int pixelWidth;		/* Total width of element in pixels (including
@@ -169,9 +169,6 @@ typedef struct {
 				 * to issue.  Malloc'ed. */
     int flags;			/* Various flag bits:  see below for
 				 * definitions. */
-    Tk_Tile tile;
-    GC tileGC;
-    Tk_TSOffset tsoffset;
 } Listbox;
 
 /*
@@ -193,22 +190,6 @@ typedef struct {
 #define UPDATE_H_SCROLLBAR	4
 #define GOT_FOCUS		8
 
-/*
- * Custom option for handling "-tile" and "-offset"
- */
-
-static Tk_CustomOption tileOption = {
-    (Tk_OptionParseProc *) Tk_TileParseProc,
-    Tk_TilePrintProc,
-    (ClientData) NULL
-};
-
-static Tk_CustomOption offsetOption = {
-    (Tk_OptionParseProc *) Tk_OffsetParseProc,
-    Tk_OffsetPrintProc,
-    (ClientData) NULL
-};
-
 /*
  * Information used for argv parsing:
  */
@@ -247,8 +228,6 @@ static Tk_ConfigSpec configSpecs[] = {
     {TK_CONFIG_PIXELS, "-highlightthickness", "highlightThickness",
 	"HighlightThickness",
 	DEF_LISTBOX_HIGHLIGHT_WIDTH, Tk_Offset(Listbox, highlightWidth), 0},
-    {TK_CONFIG_CUSTOM, "-offset", "offset", "Offset", (char *) NULL,
-	Tk_Offset(Listbox, tsoffset), TK_CONFIG_DONT_SET_DEFAULT, &offsetOption},
     {TK_CONFIG_RELIEF, "-relief", "relief", "Relief",
 	DEF_LISTBOX_RELIEF, Tk_Offset(Listbox, relief), 0},
     {TK_CONFIG_BORDER, "-selectbackground", "selectBackground", "Foreground",
@@ -272,8 +251,6 @@ static Tk_ConfigSpec configSpecs[] = {
     {TK_CONFIG_STRING, "-takefocus", "takeFocus", "TakeFocus",
 	DEF_LISTBOX_TAKE_FOCUS, Tk_Offset(Listbox, takeFocus),
 	TK_CONFIG_NULL_OK},
-    {TK_CONFIG_CUSTOM, "-tile", "tile", "Tile", (char *) NULL,
-	Tk_Offset(Listbox, tile), TK_CONFIG_DONT_SET_DEFAULT, &tileOption},
     {TK_CONFIG_INT, "-width", "width", "Width",
 	DEF_LISTBOX_WIDTH, Tk_Offset(Listbox, width), 0},
     {TK_CONFIG_STRING, "-xscrollcommand", "xScrollCommand", "ScrollCommand",
@@ -320,7 +297,7 @@ static void		ListboxLostSelection _ANSI_ARGS_((
 static void		ListboxRedrawRange _ANSI_ARGS_((Listbox *listPtr,
 			    int first, int last));
 static void		ListboxScanTo _ANSI_ARGS_((Listbox *listPtr,
-			    int x, int y, int gain));
+			    int x, int y));
 static void		ListboxSelect _ANSI_ARGS_((Listbox *listPtr,
 			    int first, int last, int select));
 static void		ListboxUpdateHScrollbar _ANSI_ARGS_((Listbox *listPtr));
@@ -331,8 +308,6 @@ static void		ListboxWorldChanged _ANSI_ARGS_((
 			    ClientData instanceData));
 static int		NearestListboxElement _ANSI_ARGS_((Listbox *listPtr,
 			    int y));
-static void		TileChangedProc _ANSI_ARGS_((ClientData clientData,
-			    Tk_Tile tile, Tk_Item *itemPtr));
 
 /*
  * The structure below defines button class behavior by means of procedures
@@ -441,11 +416,6 @@ Tk_ListboxCmd(clientData, interp, argc, argv)
     listPtr->xScrollCmd = NULL;
     listPtr->yScrollCmd = NULL;
     listPtr->flags = 0;
-    listPtr->tile = NULL;
-    listPtr->tileGC = None;
-    listPtr->tsoffset.flags = 0;
-    listPtr->tsoffset.xoffset = 0;
-    listPtr->tsoffset.yoffset = 0;
 
     Tk_SetClass(listPtr->tkwin, "Listbox");
     TkSetClassProcs(listPtr->tkwin, &listboxClass, (ClientData) listPtr);
@@ -458,7 +428,7 @@ Tk_ListboxCmd(clientData, interp, argc, argv)
 	goto error;
     }
 
-    interp->result = Tk_PathName(listPtr->tkwin);
+    Tcl_SetResult(interp, Tk_PathName(listPtr->tkwin), TCL_STATIC);
     return TCL_OK;
 
     error:
@@ -548,12 +518,14 @@ ListboxWidgetCmd(clientData, interp, argc, argv)
 	if ((index >= listPtr->topIndex) && (index < listPtr->numElements)
 		    && (index < (listPtr->topIndex + listPtr->fullLines
 		    + listPtr->partialLine))) {
+	    char buf[TCL_INTEGER_SPACE * 4];
+
 	    x = listPtr->inset + listPtr->selBorderWidth - listPtr->xOffset;
 	    y = ((index - listPtr->topIndex)*listPtr->lineHeight)
 		    + listPtr->inset + listPtr->selBorderWidth;
 	    Tk_GetFontMetrics(listPtr->tkfont, &fm);
-	    sprintf(interp->result, "%d %d %d %d", x, y, elPtr->pixelWidth,
-		    fm.linespace);
+	    sprintf(buf, "%d %d %d %d", x, y, elPtr->pixelWidth, fm.linespace);
+	    Tcl_SetResult(interp, buf, TCL_VOLATILE);
 	}
     } else if ((c == 'c') && (strncmp(argv[1], "cget", length) == 0)
 	    && (length >= 2)) {
@@ -580,7 +552,6 @@ ListboxWidgetCmd(clientData, interp, argc, argv)
     } else if ((c == 'c') && (strncmp(argv[1], "curselection", length) == 0)
 	    && (length >= 2)) {
 	int i, count;
-	char index[20];
 	Element *elPtr;
 
 	if (argc != 2) {
@@ -593,6 +564,8 @@ ListboxWidgetCmd(clientData, interp, argc, argv)
 	for (i = 0, elPtr = listPtr->firstPtr; elPtr != NULL;
 		i++, elPtr = elPtr->nextPtr) {
 	    if (elPtr->selected) {
+		char index[TCL_INTEGER_SPACE];
+
 		sprintf(index, "%d", i);
 		Tcl_AppendElement(interp, index);
 		count++;
@@ -639,8 +612,10 @@ ListboxWidgetCmd(clientData, interp, argc, argv)
 	if (GetListboxIndex(interp, listPtr, argv[2], 0, &first) != TCL_OK) {
 	    goto error;
 	}
-	if ((argc == 4) && (GetListboxIndex(interp, listPtr, argv[3],
-		0, &last) != TCL_OK)) {
+	last = first;
+	if ((argc == 4)
+		&& (GetListboxIndex(interp, listPtr, argv[3], 0,
+			&last) != TCL_OK)) {
 	    goto error;
 	}
 	if (first >= listPtr->numElements) {
@@ -657,7 +632,7 @@ ListboxWidgetCmd(clientData, interp, argc, argv)
 	if (elPtr != NULL) {
 	    if (argc == 3) {
 		if (first >= 0) {
-		    interp->result = elPtr->text;
+		    Tcl_SetResult(interp, elPtr->text, TCL_STATIC);
 		}
 	    } else {
 		for (  ; i <= last; i++, elPtr = elPtr->nextPtr) {
@@ -668,6 +643,7 @@ ListboxWidgetCmd(clientData, interp, argc, argv)
     } else if ((c == 'i') && (strncmp(argv[1], "index", length) == 0)
 	    && (length >= 3)) {
 	int index;
+	char buf[TCL_INTEGER_SPACE];
 
 	if (argc != 3) {
 	    Tcl_AppendResult(interp, "wrong # args: should be \"",
@@ -679,7 +655,8 @@ ListboxWidgetCmd(clientData, interp, argc, argv)
 		!= TCL_OK) {
 	    goto error;
 	}
-	sprintf(interp->result, "%d", index);
+	sprintf(buf, "%d", index);
+	Tcl_SetResult(interp, buf, TCL_VOLATILE);
     } else if ((c == 'i') && (strncmp(argv[1], "insert", length) == 0)
 	    && (length >= 3)) {
 	int index;
@@ -697,6 +674,7 @@ ListboxWidgetCmd(clientData, interp, argc, argv)
 	InsertEls(listPtr, index, argc-3, argv+3);
     } else if ((c == 'n') && (strncmp(argv[1], "nearest", length) == 0)) {
 	int index, y;
+	char buf[TCL_INTEGER_SPACE];
 
 	if (argc != 3) {
 	    Tcl_AppendResult(interp, "wrong # args: should be \"",
@@ -707,23 +685,21 @@ ListboxWidgetCmd(clientData, interp, argc, argv)
 	    goto error;
 	}
 	index = NearestListboxElement(listPtr, y);
-	sprintf(interp->result, "%d", index);
+	sprintf(buf, "%d", index);
+	Tcl_SetResult(interp, buf, TCL_VOLATILE);
     } else if ((c == 's') && (length >= 2)
 	    && (strncmp(argv[1], "scan", length) == 0)) {
-	int x, y, gain=10;
+	int x, y;
 
-	if ((argc != 5) && (argc != 6)) {
+	if (argc != 5) {
 	    Tcl_AppendResult(interp, "wrong # args: should be \"",
-		    argv[0], " scan mark x y\" or \"",
-		    argv[0], " scan dragto x y ?gain?\"", (char *) NULL);
+		    argv[0], " scan mark|dragto x y\"", (char *) NULL);
 	    goto error;
 	}
 	if ((Tcl_GetInt(interp, argv[3], &x) != TCL_OK)
 		|| (Tcl_GetInt(interp, argv[4], &y) != TCL_OK)) {
 	    goto error;
 	}
-	if ((argc == 6) && (Tcl_GetInt(interp, argv[5], &gain) != TCL_OK))
-            goto error;
 	if ((argv[2][0] == 'm')
 		&& (strncmp(argv[2], "mark", strlen(argv[2])) == 0)) {
 	    listPtr->scanMarkX = x;
@@ -732,7 +708,7 @@ ListboxWidgetCmd(clientData, interp, argc, argv)
 	    listPtr->scanMarkYIndex = listPtr->topIndex;
 	} else if ((argv[2][0] == 'd')
 		&& (strncmp(argv[2], "dragto", strlen(argv[2])) == 0)) {
-	    ListboxScanTo(listPtr, x, y, gain);
+	    ListboxScanTo(listPtr, x, y);
 	} else {
 	    Tcl_AppendResult(interp, "bad scan option \"", argv[2],
 		    "\": must be mark or dragto", (char *) NULL);
@@ -821,7 +797,7 @@ ListboxWidgetCmd(clientData, interp, argc, argv)
 		goto error;
 	    }
 	    if ((first < 0) || (first >= listPtr->numElements)) {
-		interp->result = "0";
+		Tcl_SetResult(interp, "0", TCL_STATIC);
 		goto done;
 	    }
 	    for (elPtr = listPtr->firstPtr, i = 0; i < first;
@@ -829,9 +805,9 @@ ListboxWidgetCmd(clientData, interp, argc, argv)
 		/* Empty loop body. */
 	    }
 	    if (elPtr->selected) {
-		interp->result = "1";
+		Tcl_SetResult(interp, "1", TCL_STATIC);
 	    } else {
-		interp->result = "0";
+		Tcl_SetResult(interp, "0", TCL_STATIC);
 	    }
 	} else if ((c == 's') && (strncmp(argv[2], "set", length) == 0)) {
 	    ListboxSelect(listPtr, first, last, 1);
@@ -843,12 +819,15 @@ ListboxWidgetCmd(clientData, interp, argc, argv)
 	}
     } else if ((c == 's') && (length >= 2)
 	    && (strncmp(argv[1], "size", length) == 0)) {
+	char buf[TCL_INTEGER_SPACE];
+
 	if (argc != 2) {
 	    Tcl_AppendResult(interp, "wrong # args: should be \"",
 		    argv[0], " size\"", (char *) NULL);
 	    goto error;
 	}
-	sprintf(interp->result, "%d", listPtr->numElements);
+	sprintf(buf, "%d", listPtr->numElements);
+	Tcl_SetResult(interp, buf, TCL_VOLATILE);
     } else if ((c == 'x') && (strncmp(argv[1], "xview", length) == 0)) {
 	int index, count, type, windowWidth, windowUnits;
 	int offset = 0;		/* Initialized to stop gcc warnings. */
@@ -858,15 +837,18 @@ ListboxWidgetCmd(clientData, interp, argc, argv)
 	    - 2*(listPtr->inset + listPtr->selBorderWidth);
 	if (argc == 2) {
 	    if (listPtr->maxWidth == 0) {
-		interp->result = "0 1";
+		Tcl_SetResult(interp, "0 1", TCL_STATIC);
 	    } else {
+		char buf[TCL_DOUBLE_SPACE * 2];
+		
 		fraction = listPtr->xOffset/((double) listPtr->maxWidth);
 		fraction2 = (listPtr->xOffset + windowWidth)
 			/((double) listPtr->maxWidth);
 		if (fraction2 > 1.0) {
 		    fraction2 = 1.0;
 		}
-		sprintf(interp->result, "%g %g", fraction, fraction2);
+		sprintf(buf, "%g %g", fraction, fraction2);
+		Tcl_SetResult(interp, buf, TCL_VOLATILE);
 	    }
 	} else if (argc == 3) {
 	    if (Tcl_GetInt(interp, argv[2], &index) != TCL_OK) {
@@ -902,15 +884,18 @@ ListboxWidgetCmd(clientData, interp, argc, argv)
 
 	if (argc == 2) {
 	    if (listPtr->numElements == 0) {
-		interp->result = "0 1";
+		Tcl_SetResult(interp, "0 1", TCL_STATIC);
 	    } else {
+		char buf[TCL_DOUBLE_SPACE * 2];
+
 		fraction = listPtr->topIndex/((double) listPtr->numElements);
 		fraction2 = (listPtr->topIndex+listPtr->fullLines)
 			/((double) listPtr->numElements);
 		if (fraction2 > 1.0) {
 		    fraction2 = 1.0;
 		}
-		sprintf(interp->result, "%g %g", fraction, fraction2);
+		sprintf(buf, "%g %g", fraction, fraction2);
+		Tcl_SetResult(interp, buf, TCL_VOLATILE);
 	    }
 	} else if (argc == 3) {
 	    if (GetListboxIndex(interp, listPtr, argv[2], 0, &index)
@@ -1004,38 +989,10 @@ DestroyListbox(memPtr)
     if (listPtr->selTextGC != None) {
 	Tk_FreeGC(listPtr->display, listPtr->selTextGC);
     }
-    if (listPtr->tile != NULL) {
-	Tk_FreeTile(listPtr->tile);
-    }
-    if (listPtr->tileGC != None) {
-	Tk_FreeGC(listPtr->display, listPtr->tileGC);
-    }
     Tk_FreeOptions(configSpecs, (char *) listPtr, listPtr->display, 0);
     ckfree((char *) listPtr);
 }
 
-/*
- *----------------------------------------------------------------------
- *
- * TileChangedProc
- *
- * Results:
- *	None.
- *
- *----------------------------------------------------------------------
- */
-/* ARGSUSED */
-static void
-TileChangedProc(clientData, tile, itemPtr)
-    ClientData clientData;
-    Tk_Tile tile;
-    Tk_Item *itemPtr;			/* Not used */
-{
-    register Listbox *listPtr = (Listbox *) clientData;
-
-    ConfigureListbox(listPtr->interp, listPtr, 0, NULL, 0);
-}
-
 /*
  *----------------------------------------------------------------------
  *
@@ -1047,7 +1004,7 @@ TileChangedProc(clientData, tile, itemPtr)
  *
  * Results:
  *	The return value is a standard Tcl result.  If TCL_ERROR is
- *	returned, then interp->result contains an error message.
+ *	returned, then the interp's result contains an error message.
  *
  * Side effects:
  *	Configuration information, such as colors, border width,
@@ -1079,6 +1036,8 @@ ConfigureListbox(interp, listPtr, argc, argv, flags)
      * background from a 3-D border.
      */
 
+    Tk_SetBackgroundFromBorder(listPtr->tkwin, listPtr->normalBorder);
+
     if (listPtr->highlightWidth < 0) {
 	listPtr->highlightWidth = 0;
     }
@@ -1122,31 +1081,12 @@ ListboxWorldChanged(instanceData)
     ClientData instanceData;	/* Information about widget. */
 {
     XGCValues gcValues;
-    GC gc = None;
+    GC gc;
     unsigned long mask;
     Listbox *listPtr;
-    Tk_Tile tile;
-    Pixmap pixmap;
 
     listPtr = (Listbox *) instanceData;
 
-    tile = listPtr->tile;
-
-    Tk_SetTileChangedProc(listPtr->tile, TileChangedProc,
-		(ClientData) listPtr, (Tk_Item *) NULL);
-
-    if((pixmap = Tk_PixmapOfTile(listPtr->tile)) != None) {
-	gcValues.fill_style = FillTiled;
-	gcValues.tile = pixmap;
-	gc = Tk_GetGC(listPtr->tkwin, GCTile|GCFillStyle, &gcValues);
-    } else {
-	Tk_SetBackgroundFromBorder(listPtr->tkwin, listPtr->normalBorder);
-    }
-    if (listPtr->tileGC != None) {
-	Tk_FreeGC(listPtr->display, listPtr->tileGC);
-    }
-    listPtr->tileGC = gc;
-
     gcValues.foreground = listPtr->fgColorPtr->pixel;
     gcValues.font = Tk_FontId(listPtr->tkfont);
     gcValues.graphics_exposures = False;
@@ -1229,38 +1169,8 @@ DisplayListbox(clientData)
 
     pixmap = Tk_GetPixmap(listPtr->display, Tk_WindowId(tkwin),
 	    Tk_Width(tkwin), Tk_Height(tkwin), Tk_Depth(tkwin));
-    if (listPtr->tileGC != NULL) {
-	if (listPtr->tsoffset.flags) {
-	    int w=0; int h=0;
-	    if (listPtr->tsoffset.flags & (TK_OFFSET_CENTER|TK_OFFSET_MIDDLE)) {
-		    Tk_SizeOfTile(listPtr->tile, &w, &h);
-	    }
-	    if (listPtr->tsoffset.flags & TK_OFFSET_LEFT) {
-		w = 0;
-	    } else if (listPtr->tsoffset.flags & TK_OFFSET_RIGHT) {
-		w = Tk_Width(tkwin);
-	    } else {
-		w = (Tk_Width(tkwin) - w) / 2;
-	    }
-	    if (listPtr->tsoffset.flags & TK_OFFSET_TOP) {
-		h = 0;
-	    } else if (listPtr->tsoffset.flags & TK_OFFSET_BOTTOM) {
-		h = Tk_Height(tkwin);
-	    } else {
-		h = (Tk_Height(tkwin) - h) / 2;
-	    }
-	    XSetTSOrigin(listPtr->display, listPtr->tileGC, w , h);
-	} else {
-	    Tk_SetTileOrigin(tkwin, listPtr->tileGC, listPtr->tsoffset.xoffset,
-		    listPtr->tsoffset.yoffset);
-	}
-	XFillRectangle(listPtr->display, pixmap, listPtr->tileGC,
-		0, 0, Tk_Width(tkwin), Tk_Height(tkwin));
-	XSetTSOrigin(listPtr->display, listPtr->tileGC, 0, 0);
-    } else {
-	Tk_Fill3DRectangle(tkwin, pixmap, listPtr->normalBorder, 0, 0,
-		Tk_Width(tkwin), Tk_Height(tkwin), 0, TK_RELIEF_FLAT);
-    }
+    Tk_Fill3DRectangle(tkwin, pixmap, listPtr->normalBorder, 0, 0,
+	    Tk_Width(tkwin), Tk_Height(tkwin), 0, TK_RELIEF_FLAT);
 
     /*
      * Iterate through all of the elements of the listbox, displaying each
@@ -1362,14 +1272,17 @@ DisplayListbox(clientData)
 	    Tk_Height(tkwin) - 2*listPtr->highlightWidth,
 	    listPtr->borderWidth, listPtr->relief);
     if (listPtr->highlightWidth > 0) {
-	GC gc;
+	GC fgGC, bgGC;
 
+	bgGC = Tk_GCForColor(listPtr->highlightBgColorPtr, pixmap);
 	if (listPtr->flags & GOT_FOCUS) {
-	    gc = Tk_GCForColor(listPtr->highlightColorPtr, pixmap);
+	    fgGC = Tk_GCForColor(listPtr->highlightColorPtr, pixmap);
+	    TkpDrawHighlightBorder(tkwin, fgGC, bgGC, 
+	            listPtr->highlightWidth, pixmap);
 	} else {
-	    gc = Tk_GCForColor(listPtr->highlightBgColorPtr, pixmap);
+	    TkpDrawHighlightBorder(tkwin, bgGC, bgGC, 
+	            listPtr->highlightWidth, pixmap);
 	}
-	Tk_DrawFocusHighlight(tkwin, gc, listPtr->highlightWidth, pixmap);
     }
     XCopyArea(listPtr->display, pixmap, Tk_WindowId(tkwin),
 	    listPtr->textGC, 0, 0, (unsigned) Tk_Width(tkwin),
@@ -1826,7 +1739,7 @@ ListboxCmdDeletedProc(clientData)
  * Results:
  *	A standard Tcl result.  If all went well, then *indexPtr is
  *	filled in with the index (into listPtr) corresponding to
- *	string.  Otherwise an error message is left in interp->result.
+ *	string.  Otherwise an error message is left in the interp's result.
  *
  * Side effects:
  *	None.
@@ -2000,13 +1913,12 @@ ChangeListboxOffset(listPtr, offset)
  */
 
 static void
-ListboxScanTo(listPtr, x, y, gain)
+ListboxScanTo(listPtr, x, y)
     register Listbox *listPtr;		/* Information about widget. */
     int x;				/* X-coordinate to use for scan
 					 * operation. */
     int y;				/* Y-coordinate to use for scan
 					 * operation. */
-    int gain;				/* gain to use for scan operation */
 {
     int newTopIndex, newOffset, maxIndex, maxOffset;
 
@@ -2043,7 +1955,7 @@ ListboxScanTo(listPtr, x, y, gain)
      * scan started.
      */
 
-    newOffset = listPtr->scanMarkXOffset - (gain*(x - listPtr->scanMarkX));
+    newOffset = listPtr->scanMarkXOffset - (10*(x - listPtr->scanMarkX));
     if (newOffset > maxOffset) {
 	newOffset = listPtr->scanMarkXOffset = maxOffset;
 	listPtr->scanMarkX = x;
